package org.bazaar_vcs.qbzr_eclipse;

import java.io.File;

import org.bazaar_vcs.qbzr_eclipse.preferences.PreferenceConstants;
import org.bazaar_vcs.qbzr_eclipse.util.StringUtilities;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The activator class controls the plug-in life cycle
 */
public class QBzrEclipsePlugin extends AbstractUIPlugin
{

    // The plug-in ID
    public static final String PLUGIN_ID = "qbzr_eclipse";

    // The shared instance
    private static QBzrEclipsePlugin plugin;

    public QBzrEclipsePlugin()
    {
        plugin = this;
    }

    @Override
    public void stop(BundleContext context) throws Exception
    {
        ResourceChangeListener.INSTANCE.stop();
        plugin = null;
        super.stop(context);
    }

    /**
     * Gets the path to the Bazaar executable that has been configured by the
     * user.
     */
    public String getBzrExecutablePath()
    {
        return getPreferenceStore().getString(PreferenceConstants.BAZAAR_PATH);
    }

    public String[] getAutoAddPatterns()
    {
        String patterns = getPreferenceStore().getString(
                PreferenceConstants.AUTO_ADD_PATTERNS);

        return StringUtilities.separateString(patterns, ";", false);
    }

    public boolean shouldLeaveDialogOpen()
    {
        return getPreferenceStore().getBoolean(
                PreferenceConstants.LEAVE_DIALOGS_OPEN);
    }

    public boolean shouldAutoAddFiles()
    {
        return getPreferenceStore().getBoolean(
                PreferenceConstants.SHOULD_AUTO_ADD_FILES);
    }

    public boolean shouldLogCommands()
    {
        return getPreferenceStore()
                .getBoolean(PreferenceConstants.LOG_COMMANDS);
    }

    public boolean shouldAutoAdd(File file)
    {
        if (!shouldAutoAddFiles() || !file.isFile())
        {
            return false;
        }

        String[] patterns = getAutoAddPatterns();
        String fileName = file.getName();

        for (String pattern : patterns)
        {
            if (fileName.matches(pattern))
            {
                return true;
            }
        }

        return false;
    }

    public static QBzrEclipsePlugin getInstance()
    {
        return plugin;
    }

    /**
     * Returns an image descriptor for the image file at the given plug-in
     * relative path
     * 
     * @param path
     *            the path
     * @return the image descriptor
     */
    public static ImageDescriptor getImageDescriptor(String path)
    {
        return imageDescriptorFromPlugin(PLUGIN_ID, path);
    }

    public void log(int severity, String message, Throwable t)
    {
        getLog()
                .log(
                        new Status(severity, "org.bazaar_vcs.qbzr_eclipse",
                                message, t));
    }

    public void log(int severity, String message)
    {
        log(severity, message, null);
    }

    public void logInfo(String message)
    {
        log(Status.INFO, message);
    }

    public void logWarning(String message)
    {
        log(Status.WARNING, message);
    }

    public void logError(String message)
    {
        log(Status.ERROR, message);
    }

    public void logException(String message, Throwable t)
    {
        log(Status.ERROR, message, t);
    }

    public void logCommand(String[] command)
    {
        if (shouldLogCommands())
        {
            logInfo("Executing command: "
                    + StringUtilities.createStringWithQuotes(command));
        }
    }
}
