package org.bazaar_vcs.qbzr_eclipse.handlers;

import java.io.File;
import java.util.List;

import org.bazaar_vcs.qbzr_eclipse.util.FileUtilities;

public class AnnotateCommandHandler extends BazaarCommandHandler
{

	@Override
	protected String getCommandName()
	{
		return "qannotate";
	}

	@Override
	protected void checkCommandCanRunOnSelection(List<File> selectedFiles,
			File fileBeingEdited,
			File branchDirectory) throws CannotRunCommandOnSelectionException
	{
		super.checkCommandCanRunOnSelection(selectedFiles, fileBeingEdited,
				branchDirectory);

		if (selectedFiles.size() > 1)
		{
			throw new CannotRunCommandOnSelectionException(
					"Annotate can only be run on one file");
		}

		if (selectedFiles.isEmpty() && fileBeingEdited == null)
		{
			throw new CannotRunCommandOnSelectionException(
					"Annotate needs to be run on a file under Bazaar version control");
		}

		File file;

		if (selectedFiles.isEmpty())
		{
			file = fileBeingEdited;
		}
		else
		{
			file = selectedFiles.get(0);
		}

		if (file.isDirectory())
		{
			throw new CannotRunCommandOnSelectionException(
					"Annotate cannot be run on a directory");
		}
	}

	@Override
	protected void addArguments(List<String> command,
			List<File> selectedFiles,
			File fileBeingEdited,
			File branchDirectory,
			File directoryToRunIn)
	{
		File file;

		if (selectedFiles.size() == 1)
		{
			file = selectedFiles.get(0);
		}
		else
		{
			file = fileBeingEdited;
		}

		assert (file != null);
		assert (branchDirectory != null);

		command.add(FileUtilities.getRelativePath(directoryToRunIn, file));
	}

}
