package org.bazaar_vcs.qbzr_eclipse.handlers;

import java.util.ArrayList;
import java.util.List;

import org.bazaar_vcs.qbzr_eclipse.util.ProcessException;
import org.bazaar_vcs.qbzr_eclipse.util.ProcessResult;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;

/** Allows multiple Bazaar commands to be executed in sequence. */

public abstract class MultiBazaarCommandHandler extends AbstractHandler
{

    public Object execute(ExecutionEvent event) throws ExecutionException
    {
        mCurrentHandlerIndex = 0;

        executeCurrentCommand(event);

        return null;
    }

    protected void addHandler(BazaarCommandHandler handler)
    {
        assert (handler != null);

        mHandlers.add(handler);
    }

    public void executeCurrentCommand(ExecutionEvent event)
            throws ExecutionException
    {
        if (mCurrentHandlerIndex >= mHandlers.size())
        {
            return;
        }

        final BazaarCommandHandler handler = mHandlers
                .get(mCurrentHandlerIndex);

        handler.setListener(new BazaarCommandHandlerListener()
        {
            public void commandSucceeded(ProcessResult result,
                    ExecutionEvent event)
            {
                handler.setListener(null);

                // Advance to the next command and execute that.

                mCurrentHandlerIndex++;

                try
                {
                    executeCurrentCommand(event);
                }
                catch (ExecutionException e)
                {
                    e.printStackTrace();
                }
            }

            public void commandAborted(ProcessResult result,
                    ExecutionEvent event)
            {
                handler.setListener(null);
            }

            public void commandCouldNotBeExecuted(ProcessException e,
                    ExecutionEvent event)
            {
                handler.setListener(null);
            }

            public void commandFailed(ProcessResult result, ExecutionEvent event)
            {
                handler.setListener(null);
            }
        });

        handler.execute(event);
    }

    private final List<BazaarCommandHandler> mHandlers = new ArrayList<BazaarCommandHandler>();

    private int mCurrentHandlerIndex;
}
