package org.bazaar_vcs.qbzr_eclipse.preferences;

import org.bazaar_vcs.qbzr_eclipse.QBzrEclipsePlugin;
import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

/**
 * This class represents a preference page that is contributed to the
 * Preferences dialog. By subclassing <samp>FieldEditorPreferencePage</samp>,
 * we can use the field support built into JFace that allows us to create a page
 * that is small and knows how to save, restore and apply itself.
 * <p>
 * This page is used to modify preferences only. They are stored in the
 * preference store that belongs to the main plug-in class. That way,
 * preferences can be accessed directly via the preference store.
 */

public class PreferencePage extends FieldEditorPreferencePage implements
		IWorkbenchPreferencePage
{

	public PreferencePage()
	{
		super(GRID);
		setPreferenceStore(QBzrEclipsePlugin.getInstance().getPreferenceStore());
		setDescription("Settings for Bazaar integration");
	}

	/**
	 * Creates the field editors. Field editors are abstractions of the common
	 * GUI blocks needed to manipulate various types of preferences. Each field
	 * editor knows how to save and restore itself.
	 */
	@Override
	public void createFieldEditors()
	{
		addField(new StringFieldEditor(PreferenceConstants.BAZAAR_PATH,
				"&Bazaar executable:", getFieldEditorParent()));
		
		addField(new BooleanFieldEditor(PreferenceConstants.LEAVE_DIALOGS_OPEN,
                "&Leave qbzr dialogs open after completion. (If possible)"
                , getFieldEditorParent()));
        
        addField(new BooleanFieldEditor(PreferenceConstants.REFRESH_PROJECTS_AFTER_FILE_CHANGING_COMMANDS,
                "&Refresh projects after running bazaar commands\nthat can change working tree (eg: update)"
                , getFieldEditorParent()));
        
        addField(new BooleanFieldEditor(PreferenceConstants.LOG_COMMANDS,
                "Log &commands"
                , getFieldEditorParent()));
		
		addField(new BooleanFieldEditor(
				PreferenceConstants.SHOULD_AUTO_ADD_FILES,
				"&Automatically add files to Bazaar\nthat match the following patterns:",
				getFieldEditorParent()));

		addField(new AutoAddPatternsListEditor(getFieldEditorParent()));
	}

	public void init(IWorkbench workbench)
	{
	}

}