package org.bazaar_vcs.qbzr_eclipse.util;

import java.io.File;

public class FileUtilities
{
	/**
	 * Gets the relative path from the supplied current directory to the
	 * supplied file. Relative paths are separated using the supplied separator.
	 * 
	 * @return a string that represents the relative path from currentDirectory
	 *         to file or null if file is not contained in currentDirectory.
	 */

	public static String getRelativePath(File file1,
			File file2,
			String sParent,
			String sSeparator)
	{
		StringBuilder buffer = new StringBuilder(100);
		File file = file1;

		if (file.isFile())
		{
			file = file.getParentFile();
		}

		while (file != null)
		{
			if (file.equals(file2) || FileUtilities.isParentOf(file, file2))
			{
				break;
			}

			buffer.append(sParent);
			buffer.append(sSeparator);
			file = file.getParentFile();
		}

		if (file == null)
		{
			return null;
		}

		boolean bAddedPath = false;
		int nInsertPos = buffer.length();

		while (!file2.equals(file))
		{
			if (bAddedPath)
			{
				buffer.insert(nInsertPos, sSeparator);
			}

			buffer.insert(nInsertPos, file2.getName());
			bAddedPath = true;
			file2 = file2.getParentFile();
		}

		return buffer.toString();
	}

	/**
	 * Gets the relative path from the supplied current directory to the
	 * supplied file. Relative paths are separated using the file separator for
	 * the current platform.
	 * 
	 * @return a string that represents the relative path from currentDirectory
	 *         to file or null if file is not contained in currentDirectory.
	 */

	public static String getRelativePath(File file1, File file2, String sParent)
	{
		return FileUtilities.getRelativePath(file1, file2, sParent,
				File.separator);
	}

	public static String getRelativePath(File file1, File file2)
	{
		return FileUtilities.getRelativePath(file1, file2, "..");
	}

	public static String getRelativeUnixPath(File file1, File file2)
	{
		return FileUtilities.getRelativePath(file1, file2, "..", "/");
	}

	public static boolean isParentOf(File file1, File file2)
	{
		File file = file2.getParentFile();

		while (file != null)
		{
			if (file.equals(file1))
			{
				return true;
			}

			file = file.getParentFile();
		}

		return false;
	}
	
	public static File getFileFromRelativePath(File base,
            String sRelativePath,
            String sParent,
            String sCurrentDir,
            String sSeparator)
    {
        if (base.isFile())
        {
            base = base.getParentFile();
        }

        final String[] paths = StringUtilities.separateString(sRelativePath,
                sSeparator, false);
        File file = base;

        for (int i = 0; i < paths.length; i++)
        {
            final String sPath = paths[i];

            if (i == paths.length - 1 && sPath.length() == 0)
            {
                break;
            }

            if (sPath.equals(sParent))
            {
                file = file.getParentFile();
            }
            else if (sPath.equals(sCurrentDir))
            {
            }
            else
            {
                file = new File(file, sPath);
            }
        }

        return file;
    }

    public static File getFileFromRelativePath(File base,
            String sRelativePath,
            String sParent,
            String sCurrentDir)
    {
        return FileUtilities.getFileFromRelativePath(base, sRelativePath,
                sParent, sCurrentDir, File.separator);
    }

    public static File getFileFromRelativePath(File base, String sRelativePath)
    {
        return FileUtilities.getFileFromRelativePath(base, sRelativePath, "..",
                ".", File.separator);
    }

    public static File getFileFromRelativeUnixPath(File base,
            String relativePath)
    {
        return getFileFromRelativePath(base, relativePath, "..", ".", "/");
    }
}
