package org.bazaar_vcs.qbzr_eclipse.util;

import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;

public final class ProcessUtilities
{
	private ProcessUtilities()
	{
	}

	/**
	 * Executes the given command in the given directory and returns the exit
	 * code and any error messages output by the command.
	 * 
	 * @throws ProcessException
	 */
	public static ProcessResult exec(String[] command, File directory)
			throws ProcessException
	{
		final Process process;
		final StringBuilder errorMessages;
		final StringBuilder outputMessages;
		try
		{
			process = Runtime.getRuntime().exec(command, null, directory);
			errorMessages = new StringBuilder();

			Thread errorReader = new Thread()
			{
				@Override
				public void run()
				{
					Reader input = new InputStreamReader(process
							.getErrorStream());
					int c;

					try
					{
						while ((c = input.read()) != -1)
						{
							errorMessages.append((char) c);
						}
					}
					catch (IOException e)
					{
					}
				}
			};

			errorReader.start();

			outputMessages = new StringBuilder();

			Thread outputReader = new Thread()
			{
				@Override
				public void run()
				{
					Reader input = new InputStreamReader(process
							.getInputStream());
					int c;

					try
					{
						while ((c = input.read()) != -1)
						{
							outputMessages.append((char) c);
						}
					}
					catch (IOException e)
					{
					}
				}
			};

			outputReader.start();

			try
			{
				process.waitFor();
			}
			catch (InterruptedException e)
			{
				throw new ProcessException(command, e);
			}

			try
			{
				errorReader.join();
			}
			catch (InterruptedException e)
			{
			}

			try
			{
				outputReader.join();
			}
			catch (InterruptedException e)
			{
			}

			return new ProcessResult(command, process.exitValue(),
					outputMessages.toString(), errorMessages.toString());
		}
		catch (IOException e)
		{
			throw new ProcessException(command, e);
		}
	}

	public static ProcessResult exec(String command, File directory)
			throws ProcessException
	{
		return exec(StringUtilities.separateStringWithQuotes(command),
				directory);
	}
}
