package org.bazaar_vcs.qbzr_eclipse.util;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

public class StringUtilities
{
	/**
	 * Separates the supplied string into separate strings which are separated
	 * by the separator. The separated strings can be automatically trimmed
	 * (remove white space from start and end. e.g. separateString (" 1 , 2, 3,
	 * 4", ",", true) returns {"1","2","3","4"}.
	 * 
	 * @param string
	 *            the string to be separated
	 * @param separator
	 *            the string used as the separator (e.g. ",")
	 * @param trim
	 *            true if white space should be trimmed from the separated
	 *            strings or false if they shouldn't
	 */

	public static String[] separateString(String string,
			String separator,
			boolean trim)
	{
		// Calculate number of separator strings

		List<String> list = new LinkedList<String>();
		int i = 0;

		while (true)
		{
			int index = string.indexOf(separator, i);

			if (index == -1)
			{
				list.add(string.substring(i));
				break;
			}

			list.add(string.substring(i, index));
			i = index + separator.length();
		}

		String[] result = new String[list.size()];
		Iterator<String> iterator = list.iterator();

		for (i = 0; iterator.hasNext(); i++)
		{
			String s = (String) iterator.next();
			result[i] = trim ? s.trim() : s;
		}

		return result;
	}

	/**
	 * Separates the given string into elements that are separated by white
	 * space. An element may contain white space if it is surrounded with '"'
	 * character.
	 * 
	 * @param command
	 * @return
	 */
	public static String[] separateStringWithQuotes(String command)
	{
		List<String> result = new ArrayList<String>();
		StringBuilder buffer = null;

		for (int i = 0; i < command.length(); i++)
		{
			char c = command.charAt(i);

			if (Character.isWhitespace(c))
			{
				if (buffer != null)
				{
					result.add(buffer.toString());
					buffer = null;
				}

				continue;
			}

			buffer = new StringBuilder();

			if (c == '"')
			{
				i++;

				for (; i < command.length(); i++)
				{
					c = command.charAt(i);

					if (c == '"')
					{
						i++;
						break;
					}

					buffer.append(c);
				}
			}
		}

		if (buffer != null)
		{
			result.add(buffer.toString());
		}

		return result.toArray(new String[result.size()]);
	}

	public static String createStringWithQuotes(String[] elements)
	{
		StringBuilder buffer = new StringBuilder();

		for (String element : elements)
		{
			boolean hasWhiteSpace = false;

			for (int i = 0; i < element.length(); i++)
			{
				char c = element.charAt(i);

				if (Character.isWhitespace(c))
				{
					hasWhiteSpace = true;
					break;
				}
			}

			if (buffer.length() > 0)
			{
				buffer.append(' ');
			}

			if (hasWhiteSpace)
			{
				buffer.append('"');
				buffer.append(element);
				buffer.append('"');
			}
			else
			{
				buffer.append(element);
			}
		}

		return buffer.toString();
	}
}
